from .base_model import Model
from .sklean_model import SkModel
from .keras_model import KerasModel
# from .multi_model import
from .pytoch_model import PytorchModel
from .reinforcement_model import ReinforcementModel
from .auto_encoder_model import AutoEncoderModel
